import { requestAPI } from '../server';
import { ErrorTypeKeys, ReportIds, } from '../linter/interfaces';
import { IExperimentConfigAttributes, } from './experimentmanager';
export class Config {
    constructor(experimentConfig, status, eh) {
        this._eh = eh;
        this.defaultOptions = this.createDefault();
        this.experimentConfig = experimentConfig;
        this.status = status;
        this.load();
    }
    createDefault() {
        return {
            mode: 'type',
            requirements: 'requirements.txt',
            types: {
                invalidtitle: true,
                hiddenstate: true,
                confusenotebook: true,
                import: true,
                absolutepath: true,
            },
            reports: {
                c1: true,
                c2: true,
                c3: true,
                c4: true,
                c5: true,
                h1: true,
                h2: true,
                h3: true,
                h4: true,
                h5: true,
                h6: true,
                i1: true,
                i2: true,
                p1: true,
                t1: true,
                t2: true,
                t3: true,
                t4: true,
                t5: true,
                t6: true,
                t7: true,
            },
        };
    }
    loadData(options, experiment, data) {
        /* eslint @typescript-eslint/explicit-module-boundary-types: 0 */
        try {
            if (options) {
                this.merge(options, data.options);
            }
            if (experiment) {
                const dataExperiment = data.experiment;
                if (dataExperiment) {
                    for (const key of IExperimentConfigAttributes) {
                        const value = dataExperiment[key];
                        if (value !== undefined && value !== null) {
                            experiment[key] = value;
                        }
                    }
                }
                else {
                    experiment.enabled = false;
                }
                this.status.experiment = experiment.enabled;
            }
            else {
                this.status.experiment = false;
            }
        }
        catch (error) {
            throw this._eh.report(error, 'Config:loadData', [
                options,
                experiment,
                data,
            ]);
        }
    }
    load(onSuccess, onError) {
        try {
            requestAPI('config')
                .then((data) => {
                this.loadData(this.defaultOptions, this.experimentConfig, data);
                if (onSuccess) {
                    onSuccess(data);
                }
            })
                .catch((reason) => {
                this.status.serverSide = false;
                this._eh.report(`The julynter server extension appears to be missing.\n${reason}`, 'Config:loadData', []);
                if (onError) {
                    onError(reason);
                }
            });
        }
        catch (error) {
            throw this._eh.report(error, 'Config:load', []);
        }
    }
    loadUser() {
        try {
            return requestAPI('userconfig').then((data) => {
                const config = this.createDefault();
                this.loadData(config, null, data);
                return config;
            });
        }
        catch (error) {
            throw this._eh.report(error, 'Config:loadUser', []);
        }
    }
    loadProject() {
        try {
            return requestAPI('config').then((data) => {
                const config = this.createDefault();
                this.loadData(config, null, data);
                this.loadData(this.defaultOptions, this.experimentConfig, data);
                return config;
            });
        }
        catch (error) {
            throw this._eh.report(error, 'Config:loadProject', []);
        }
    }
    saveProject(options) {
        try {
            this.defaultOptions = options;
            return requestAPI('config', {
                body: JSON.stringify({ options: options }),
                method: 'POST',
            });
        }
        catch (error) {
            throw this._eh.report(error, 'Config:saveProject', [options]);
        }
    }
    saveUser(options) {
        try {
            return requestAPI('userconfig', {
                body: JSON.stringify({ options: options }),
                method: 'POST',
            }).then((data) => {
                this.load();
                return data;
            });
        }
        catch (error) {
            throw this._eh.report(error, 'Config:saveUser', [options]);
        }
    }
    merge(original, newOptions) {
        try {
            if (!newOptions) {
                return;
            }
            if (newOptions.mode !== undefined) {
                original.mode = newOptions.mode;
            }
            if (newOptions.requirements !== undefined) {
                original.requirements = newOptions.requirements;
            }
            if (newOptions.types !== undefined) {
                for (const key of ErrorTypeKeys) {
                    const value = newOptions.types[key];
                    if (value !== undefined) {
                        original.types[key] = value;
                    }
                }
            }
            if (newOptions.reports !== undefined) {
                for (const key of ReportIds) {
                    const value = newOptions.reports[key];
                    if (value !== undefined) {
                        original.reports[key] = value;
                    }
                }
            }
        }
        catch (error) {
            throw this._eh.report(error, 'Config:merge', [original, newOptions]);
        }
    }
}
