import { Clipboard } from '@jupyterlab/apputils';
import { CodeCell } from '@jupyterlab/cells';
import { NotebookActions } from '@jupyterlab/notebook';
import { requestAPI } from '../server';
const JUPYTER_CELL_MIME = 'application/vnd.jupyter.cells';
export const IExperimentConfigAttributes = [
    'lintingMessages',
    'lintingTypes',
    'execution',
    'activity',
    'code',
    'name',
    'enabled',
];
export class ExperimentManager {
    constructor(docmanager) {
        this.config = {
            lintingMessages: 'maybe',
            lintingTypes: 'maybe',
            execution: 'maybe',
            activity: 'maybe',
            code: 'maybe',
            name: 'maybe',
            enabled: true,
        };
        this.lastLint = {};
        this._overrideJupyterLab(docmanager);
    }
    _overrideJupyterLab(docmanager) {
        const oldRename = docmanager.rename.bind(docmanager);
        docmanager.rename = (oldPath, newPath) => {
            return oldRename(oldPath, newPath).then((result) => {
                this.reportRename(oldPath, newPath);
                return result;
            });
        };
        const oldExecute = CodeCell.execute;
        CodeCell['execute'] = (cell, sessionContext, metadata) => {
            return oldExecute(cell, sessionContext, metadata).then((msg) => {
                this.reportExecution(cell, sessionContext);
                return msg;
            });
        };
        const oldSplit = NotebookActions.splitCell;
        NotebookActions.splitCell = (notebook) => {
            const oldIndex = notebook.activeCellIndex;
            const result = oldSplit(notebook);
            this.reportSplitCell(oldIndex, notebook);
            return result;
        };
        const oldMerge = NotebookActions.mergeCells;
        NotebookActions.mergeCells = (notebook) => {
            const toMerge = this._getActiveIndexes(notebook);
            const result = oldMerge(notebook);
            this.reportMergeCell(toMerge, notebook);
            return result;
        };
        const oldDeleteCells = NotebookActions.deleteCells;
        NotebookActions.deleteCells = (notebook) => {
            const toDelete = this._getActiveIndexes(notebook);
            const result = oldDeleteCells(notebook);
            this.reportIndexesAction(toDelete, notebook, 'deleteCells');
            return result;
        };
        const oldInsertAbove = NotebookActions.insertAbove;
        NotebookActions.insertAbove = (notebook) => {
            const oldIndex = notebook.activeCellIndex;
            const result = oldInsertAbove(notebook);
            this.reportIndexesAction([oldIndex], notebook, 'insertAbove');
            return result;
        };
        const oldInsertBelow = NotebookActions.insertBelow;
        NotebookActions.insertBelow = (notebook) => {
            const oldIndex = notebook.activeCellIndex;
            const result = oldInsertBelow(notebook);
            this.reportIndexesAction([oldIndex], notebook, 'insertBelow');
            return result;
        };
        const oldMoveDown = NotebookActions.moveDown;
        NotebookActions.moveDown = (notebook) => {
            const toMove = this._getActiveIndexes(notebook);
            const result = oldMoveDown(notebook);
            this.reportIndexesAction(toMove, notebook, 'moveDown');
            return result;
        };
        const oldMoveUp = NotebookActions.moveUp;
        NotebookActions.moveUp = (notebook) => {
            const toMove = this._getActiveIndexes(notebook);
            const result = oldMoveUp(notebook);
            this.reportIndexesAction(toMove, notebook, 'moveUp');
            return result;
        };
        const oldChangeCellType = NotebookActions.changeCellType;
        NotebookActions.changeCellType = (notebook, value) => {
            const index = notebook.activeCellIndex;
            const oldValue = notebook.activeCell.model.type;
            const result = oldChangeCellType(notebook, value);
            this.reportChangeCellType(index, oldValue, value, notebook);
            return result;
        };
        const oldRun = NotebookActions.run;
        NotebookActions.run = (notebook, sessionContext) => {
            const cell = notebook.activeCell;
            if (cell.model.type === 'markdown') {
                this.reportExecution(cell, sessionContext);
            }
            const result = oldRun(notebook, sessionContext);
            return result;
        };
        const oldRunAndAdvance = NotebookActions.runAndAdvance;
        NotebookActions.runAndAdvance = (notebook, sessionContext) => {
            const index = notebook.activeCellIndex;
            const last = notebook.widgets.length - 1;
            const cell = notebook.activeCell;
            if (cell.model.type === 'markdown') {
                this.reportExecution(cell, sessionContext);
            }
            const result = oldRunAndAdvance(notebook, sessionContext);
            if (index === last) {
                this.reportIndexesAction([index], notebook, 'newCellAfterRun');
            }
            return result;
        };
        const oldRunAndInsert = NotebookActions.runAndInsert;
        NotebookActions.runAndInsert = (notebook, sessionContext) => {
            const index = notebook.activeCellIndex;
            const cell = notebook.activeCell;
            if (cell.model.type === 'markdown') {
                this.reportExecution(cell, sessionContext);
            }
            const result = oldRunAndInsert(notebook, sessionContext);
            this.reportIndexesAction([index], notebook, 'newCellAfterRun');
            return result;
        };
        const oldCut = NotebookActions.cut;
        NotebookActions.cut = (notebook) => {
            const toDelete = this._getActiveIndexes(notebook);
            const result = oldCut(notebook);
            this.reportIndexesAction(toDelete, notebook, 'cut');
            return result;
        };
        const oldPaste = NotebookActions.paste;
        NotebookActions.paste = (notebook, mode = 'below') => {
            if (!notebook.model || !notebook.activeCell) {
                return oldPaste(notebook, mode);
            }
            const clipboard = Clipboard.getInstance();
            if (!clipboard.hasData(JUPYTER_CELL_MIME)) {
                return oldPaste(notebook, mode);
            }
            const toInsert = clipboard.getData(JUPYTER_CELL_MIME);
            let toDelete = [];
            if (mode === 'replace') {
                toDelete = this._getActiveIndexes(notebook);
            }
            const index = notebook.activeCellIndex;
            const result = oldPaste(notebook, mode);
            this.reportPaste(index, toInsert, toDelete, mode, notebook);
            return result;
        };
        const oldUndo = NotebookActions.undo;
        NotebookActions.undo = (notebook) => {
            const result = oldUndo(notebook);
            this.reportAction(notebook, 'undo');
            return result;
        };
        const oldRedo = NotebookActions.redo;
        NotebookActions.redo = (notebook) => {
            const result = oldRedo(notebook);
            this.reportAction(notebook, 'redo');
            return result;
        };
        const oldRunAll = NotebookActions.runAll;
        NotebookActions.runAll = (notebook, sessionContext) => {
            this.reportAction(notebook, 'runAll');
            return oldRunAll(notebook, sessionContext);
        };
        const oldRunAllAbove = NotebookActions.runAllAbove;
        NotebookActions.runAllAbove = (notebook, sessionContext) => {
            this.reportAction(notebook, 'runAllAbove');
            return oldRunAllAbove(notebook, sessionContext);
        };
        const oldRunAllBelow = NotebookActions.runAllBelow;
        NotebookActions.runAllBelow = (notebook, sessionContext) => {
            this.reportAction(notebook, 'runAllBelow');
            return oldRunAllBelow(notebook, sessionContext);
        };
        const oldClearOutputs = NotebookActions.clearOutputs;
        NotebookActions.clearOutputs = (notebook) => {
            const toDelete = this._getActiveIndexes(notebook);
            const result = oldClearOutputs(notebook);
            this.reportIndexesAction(toDelete, notebook, 'clearOutputs');
            return result;
        };
        const oldClearAllOutputs = NotebookActions.clearAllOutputs;
        NotebookActions.clearAllOutputs = (notebook) => {
            this.reportAction(notebook, 'clearAllOutputs');
            return oldClearAllOutputs(notebook);
        };
    }
    /* Start Actions */
    reportRename(oldName, newName) {
        if (!this.config.enabled || !this.config.activity) {
            return;
        }
        const send = {
            header: 'Action',
            operation: 'rename',
            oldName: oldName,
            newName: newName,
        };
        this._send(send);
    }
    reportSplitCell(oldIndex, notebook) {
        if (!this.config.enabled || !this.config.activity) {
            return;
        }
        const newIndex = notebook.activeCellIndex;
        let codes = null;
        if (this.config.code) {
            codes = [];
            for (let i = oldIndex; i <= newIndex; i++) {
                codes.push(notebook.widgets[i].model.value.text);
            }
        }
        const send = {
            header: 'Action',
            operation: 'splitCell',
            notebookName: this._notebookName(notebook.title.label),
            notebookId: notebook.parent.id,
            indexes: [oldIndex],
            newIndexes: [newIndex],
            codes: codes,
        };
        this._send(send);
    }
    reportMergeCell(toMerge, notebook) {
        if (!this.config.enabled || !this.config.activity) {
            return;
        }
        const newIndex = notebook.activeCellIndex;
        let code = null;
        if (this.config.code) {
            code = notebook.widgets[newIndex].model.value.text;
        }
        const send = {
            header: 'Action',
            operation: 'mergeCells',
            notebookName: this._notebookName(notebook.title.label),
            notebookId: notebook.parent.id,
            indexes: toMerge,
            newIndexes: [newIndex],
            codes: [code],
        };
        this._send(send);
    }
    reportIndexesAction(indexes, notebook, operation) {
        if (!this.config.enabled || !this.config.activity) {
            return;
        }
        const send = {
            header: 'Action',
            operation: operation,
            notebookName: this._notebookName(notebook.title.label),
            notebookId: notebook.parent.id,
            indexes: indexes,
        };
        this._send(send);
    }
    reportChangeCellType(index, oldValue, newValue, notebook) {
        if (!this.config.enabled || !this.config.activity) {
            return;
        }
        const send = {
            header: 'Action',
            operation: 'changeCellType',
            notebookName: this._notebookName(notebook.title.label),
            notebookId: notebook.parent.id,
            indexes: [index],
            oldType: oldValue,
            newType: newValue,
        };
        this._send(send);
    }
    reportPaste(index, toInsert, toDelete, mode, notebook) {
        if (!this.config.enabled || !this.config.activity) {
            return;
        }
        let codes = null;
        if (this.config.code) {
            codes = [];
            for (let i = 0; i < toInsert.length; i++) {
                if (Array.isArray(toInsert[i].source)) {
                    codes.push(toInsert[i].source.join('\n'));
                }
                else {
                    codes.push(toInsert[i].source);
                }
            }
        }
        const send = {
            header: 'Action',
            operation: 'paste',
            notebookName: this._notebookName(notebook.title.label),
            notebookId: notebook.parent.id,
            indexes: toDelete,
            newIndexes: [index],
            codes: codes,
            info: mode,
            pasteCount: toInsert.length,
        };
        this._send(send);
    }
    reportAction(notebook, operation) {
        if (!this.config.enabled || !this.config.activity) {
            return;
        }
        const send = {
            header: 'Action',
            operation: operation,
            notebookName: this._notebookName(notebook.title.label),
            notebookId: notebook.parent.id,
        };
        this._send(send);
    }
    /* End Actions */
    /* Start Activities */
    reportLoadConfig(nbPanel, checks) {
        if (!this.config.enabled || !this.config.activity) {
            return;
        }
        const send = {
            header: 'Activity',
            operation: 'loadConfig',
            notebookName: this._notebookName(nbPanel.title.label),
            notebookId: nbPanel.id,
            options: checks,
        };
        this._send(send);
    }
    reportSaveConfig(nbPanel, checks) {
        if (!this.config.enabled || !this.config.activity) {
            return;
        }
        const send = {
            header: 'Activity',
            operation: 'saveConfig',
            notebookName: this._notebookName(nbPanel.title.label),
            notebookId: nbPanel.id,
            options: checks,
        };
        this._send(send);
    }
    reportCloseNotebook(handler) {
        if (!this.config.enabled || !this.config.activity) {
            return;
        }
        if ({}.hasOwnProperty.call(this.lastLint, handler.id)) {
            delete this.lastLint[handler.id];
        }
        const send = {
            header: 'Activity',
            operation: 'close',
            notebookName: this._notebookName(handler.name),
            notebookId: handler.id,
        };
        this._send(send);
    }
    reportKernelActivity(handler, operation, kernelName) {
        if (!this.config.enabled || !this.config.activity) {
            return;
        }
        const send = {
            header: 'Activity',
            operation: operation,
            notebookName: this._notebookName(handler.name),
            notebookId: handler.id,
            info: kernelName,
        };
        this._send(send);
    }
    reportActivity(handler, operation) {
        if (!this.config.enabled || !this.config.activity) {
            return;
        }
        const send = {
            header: 'Activity',
            operation: operation,
            notebookName: this._notebookName(handler.name),
            notebookId: handler.id,
        };
        this._send(send);
    }
    reportVisibility(handler, visible) {
        if (!this.config.enabled || !this.config.activity) {
            return;
        }
        if (!handler) {
            return;
        }
        const send = {
            header: 'Activity',
            operation: 'visibility',
            notebookName: this._notebookName(handler.name),
            notebookId: handler.id,
            info: visible.toString(),
        };
        this._send(send);
    }
    reportSetConfig(nbPanel, config, value) {
        if (!this.config.enabled || !this.config.activity) {
            return;
        }
        const send = {
            header: 'Activity',
            operation: 'setConfig',
            notebookName: this._notebookName(nbPanel.title.label),
            notebookId: nbPanel.id,
            param: config,
            info: value.toString(),
        };
        this._send(send);
    }
    reportNotebookKernel(handler, kernel, language) {
        if (!this.config.enabled || !this.config.activity) {
            return;
        }
        const send = {
            header: 'Activity',
            operation: 'loadKernel',
            notebookName: this._notebookName(handler.name),
            notebookId: handler.id,
            param: language,
            info: kernel,
        };
        this._send(send);
    }
    /* End Activities */
    /* Start code */
    reportNotebookCode(handler) {
        if (!this.config.enabled || !(this.config.code || this.config.execution)) {
            return;
        }
        const cells = [];
        const nbcells = handler.nbPanel.content.widgets;
        for (let i = 0; i < nbcells.length; i++) {
            const cell = nbcells[i];
            const cellResult = this._collectCell(cell);
            cells.push(cellResult);
        }
        const send = {
            header: 'Code',
            operation: 'code',
            notebookName: this._notebookName(handler.name),
            notebookId: handler.id,
            cells: cells,
        };
        this._send(send);
    }
    reportExecution(cell, sessionContext) {
        if (!this.config.enabled || !(this.config.code || this.config.execution)) {
            return;
        }
        const send = {
            header: 'Execution',
            operation: 'execute',
            notebookName: this._notebookName(sessionContext.path),
            notebookId: cell.parent.parent.id,
            cells: [this._collectCell(cell)],
            indexes: [cell.parent.widgets.indexOf(cell)],
        };
        this._send(send);
    }
    /* End code */
    /* Start lint */
    reportLinting(handler, reports) {
        if (!this.config.enabled) {
            return;
        }
        if (!{}.hasOwnProperty.call(this.lastLint, handler.id)) {
            this.lastLint[handler.id] = [];
        }
        const newReports = reports.filter((report) => {
            if (report.type === 'group') {
                return false;
            }
            const same = this.lastLint[handler.id].find((other) => {
                return (other.text === report.text &&
                    other.reportType === report.reportType &&
                    other.cellId === report.cellId &&
                    other.reportId === report.reportId);
            });
            if (same) {
                report.feedback = same.feedback;
                same.kept = true;
                return false;
            }
            else {
                report.feedback = 1;
            }
            return true;
        });
        const removedReports = this.lastLint[handler.id].filter((report) => {
            if (report.type === 'group') {
                return false;
            }
            if (report.kept) {
                return false;
            }
            return true;
        });
        this.lastLint[handler.id] = reports;
        if (!(this.config.lintingMessages || this.config.lintingTypes)) {
            return;
        }
        if (newReports.length === 0 && removedReports.length === 0) {
            return;
        }
        const mapfn = this.config.lintingMessages
            ? this.selectMessages.bind(this)
            : this.selectTypes.bind(this);
        const send = {
            header: 'Lint',
            operation: 'lint',
            notebookName: this._notebookName(handler.name),
            notebookId: handler.id,
            newReports: newReports.map(mapfn),
            removedReports: removedReports.map(mapfn),
        };
        this._send(send);
    }
    reportFeedback(handler, report, message) {
        if (!this.config.enabled) {
            return;
        }
        const mapfn = this.config.lintingMessages
            ? this.selectMessages.bind(this)
            : this.selectTypes.bind(this);
        const newReport = mapfn(report);
        const send = {
            header: 'Lint',
            operation: 'feedback',
            notebookName: this._notebookName(handler.name),
            notebookId: handler.id,
            report: newReport,
            message: message,
        };
        this._send(send);
    }
    reportLintClick(handler, report) {
        if (!this.config.enabled || !this.config.activity) {
            return;
        }
        if (!this.config.lintingMessages && !this.config.lintingTypes) {
            return;
        }
        const mapfn = this.config.lintingMessages
            ? this.selectMessages.bind(this)
            : this.selectTypes.bind(this);
        const newReport = mapfn(report);
        const send = {
            header: 'Lint',
            operation: 'lintclick',
            notebookName: this._notebookName(handler.name),
            notebookId: handler.id,
            report: newReport,
        };
        this._send(send);
    }
    selectMessages(report) {
        return {
            text: report.text,
            reportType: report.reportType,
            reportId: report.reportId,
            suggestion: report.suggestion,
            cellId: report.cellId.toString(),
            visible: report.visible,
            filteredOut: report.filteredOut,
            type: report.type,
        };
    }
    selectTypes(report) {
        return {
            reportType: report.reportType,
            reportId: report.reportId,
            cellId: report.cellId.toString(),
            visible: report.visible,
            filteredOut: report.filteredOut,
            type: report.type,
        };
    }
    /* End lint */
    _collectCell(cell) {
        const model = cell.model;
        const cellResult = {
            type: model.type,
            code: this.config.code ? model.value.text : null,
            length: model.value.text.length,
            empty: model.value.text.trim().length === 0,
            executionCount: null,
            outputs: null,
        };
        if (this.config.execution && model.type === 'code') {
            const outputs = [];
            const codeModel = model;
            const codeOutputs = codeModel.outputs.toJSON();
            for (let j = 0; j < codeOutputs.length; j++) {
                const codeOutput = codeOutputs[j];
                const output = {
                    type: codeOutput.output_type,
                    mime: [],
                };
                if (output.type === 'stream') {
                    output.mime = [codeOutput.name];
                }
                else if (output.type === 'error') {
                    output.mime = [codeOutput.ename];
                }
                else if ({}.hasOwnProperty.call(codeOutput, 'data')) {
                    output.mime = Object.keys(codeOutput.data);
                }
                outputs.push(output);
            }
            cellResult.executionCount = codeModel.executionCount;
            cellResult.outputs = outputs;
        }
        return cellResult;
    }
    _getActiveIndexes(notebook) {
        const result = [];
        notebook.widgets.forEach((child, index) => {
            if (notebook.isSelectedOrActive(child)) {
                result.push(index);
            }
        });
        return result;
    }
    _notebookName(name) {
        return this.config.name ? name : '<redacted>';
    }
    _send(data) {
        data['date'] = new Date();
        return requestAPI('experiment', {
            body: JSON.stringify(data),
            method: 'POST',
        }).catch((reason) => {
            console.error(`The julynter server extension appears to be missing.\n${reason}`);
            return reason;
        });
    }
}
