import { showDialog, Dialog } from '@jupyterlab/apputils';
import { renameDialog } from '@jupyterlab/docmanager';
import { ERRORS } from '../linter/reports';
function isNumber(value) {
    return value !== null && !isNaN(Number(value.toString()));
}
function capitalizeFirstLetter(string) {
    return string.charAt(0).toUpperCase() + string.slice(1);
}
export class ItemGenerator {
    constructor(docManager, handler, eh) {
        this._docManager = docManager;
        this._handler = handler;
        this._notebookContent = handler.nbPanel.content;
        this._eh = eh;
    }
    create(cellId, type, messageId, args) {
        try {
            const message = ERRORS[messageId];
            return {
                text: message.label(...args),
                reportType: message.type,
                reportId: messageId,
                suggestion: message.suggestion,
                cellId: cellId,
                visible: true,
                filteredOut: false,
                type: type,
                action: message.action,
                boundAction: message.action.execute(this, ...args),
            };
        }
        catch (error) {
            throw this._eh.report(error, 'ItemGenerator:create', [
                cellId,
                type,
                messageId,
                args,
            ]);
        }
    }
    renameNotebook() {
        try {
            renameDialog(this._docManager, this._handler.nbPanel.context.path);
        }
        catch (error) {
            throw this._eh.report(error, 'ItemGenerator:renameNotebook', [
                this._handler.nbPanel.context.path,
            ]);
        }
    }
    goToCell(index) {
        try {
            const cell = this._notebookContent.widgets[index];
            this._notebookContent.activeCellIndex = index;
            cell.node.scrollIntoView();
        }
        catch (error) {
            throw this._eh.report(error, 'ItemGenerator:goToCell', [index]);
        }
    }
    addModule(index, module) {
        try {
            const handler = this._handler;
            showDialog({
                title: 'Add requirement',
                body: `Add "${module}" to requirements?`,
                buttons: [Dialog.cancelButton(), Dialog.okButton({ label: 'Add' })],
            }).then((result) => {
                Promise.resolve(result.button.accept).then((ok) => {
                    try {
                        if (ok) {
                            handler.addModule(module);
                        }
                    }
                    catch (error) {
                        throw this._eh.report(error, 'ItemGenerator:addModule.ok', [
                            index,
                            module,
                            ok,
                        ]);
                    }
                });
            });
            this.goToCell(index);
        }
        catch (error) {
            throw this._eh.report(error, 'ItemGenerator:addModule', [index, module]);
        }
    }
    restoreCell(index, executionCount, code) {
        try {
            const cell = this._notebookContent.model.contentFactory.createCodeCell({});
            cell.value.text = code;
            cell.executionCount = executionCount;
            this._notebookContent.model.cells.insert(index, cell);
            this.goToCell(index);
        }
        catch (error) {
            throw this._eh.report(error, 'ItemGenerator:restoreCell', [
                index,
                executionCount,
                code,
            ]);
        }
    }
}
export class GroupGenerator {
    constructor(nbPanel, update, eh) {
        this._nbPanel = nbPanel;
        this._update = update;
        this._eh = eh;
    }
    create(title, reportType, elements) {
        try {
            let strTitle;
            let metavar;
            let metaname;
            if (isNumber(title)) {
                strTitle = 'Cell ' + title;
                const cell = this._nbPanel.content.widgets[Number(title)];
                metavar = cell.model.metadata;
                metaname = 'julynter-cellgroup-collapsed';
            }
            else {
                strTitle = capitalizeFirstLetter(String(title));
                metavar = this._nbPanel.model.metadata;
                metaname =
                    'julynter-cellgroup-' +
                        strTitle.replace(' ', '-').toLowerCase() +
                        '-collapsed';
            }
            let collapsed = metavar.get(metaname);
            collapsed = collapsed !== undefined ? collapsed : false;
            elements.forEach((element) => {
                element.visible = !collapsed;
                element.hasParent = true;
            });
            const result = {
                text: strTitle,
                reportType: reportType,
                reportId: 'group',
                cellId: 'group',
                suggestion: null,
                visible: true,
                filteredOut: false,
                collapsed: collapsed,
                type: 'group',
                hasParent: true,
                action: null,
                boundAction: () => {
                    return;
                },
            };
            const onClickFactory = (line) => {
                return () => {
                    try {
                        elements.forEach((element) => {
                            element.visible = result.collapsed;
                        });
                        result.collapsed = !result.collapsed;
                        result.action.label = result.collapsed
                            ? 'Expand category'
                            : 'Collapse category';
                        metavar.set(metaname, !collapsed);
                        this._update();
                    }
                    catch (error) {
                        throw this._eh.report(error, 'GroupGenerator:create.click', [
                            title,
                            reportType,
                            elements,
                        ]);
                    }
                };
            };
            result.boundAction = onClickFactory(0);
            result.action = {
                label: result.collapsed ? 'Expand category' : 'Collapse category',
                title: 'Collapse or expand lints of this type',
                execute: (ItemGenerator, ...args) => {
                    return () => {
                        return;
                    };
                },
            };
            return result;
        }
        catch (error) {
            throw this._eh.report(error, 'GroupGenerator:create', [
                title,
                reportType,
                elements,
            ]);
        }
    }
}
