import * as React from 'react';
import { CommandRegistry } from '@lumino/commands';
import { ContextMenu } from '@lumino/widgets';
import { Clipboard, Dialog, showDialog } from '@jupyterlab/apputils';
import { FeedbackDialogRenderer, showTextDialog, } from './feedbackrenderer'; /* eslint @typescript-eslint/no-unused-vars: 0 */
export class ItemRenderer extends React.Component {
    constructor(props) {
        super(props);
        this.handleClick = this.handleClick.bind(this);
    }
    doClick() {
        try {
            const notebook = this.props.notebook;
            notebook.experimentManager.reportLintClick(notebook, this.props.item);
            this.props.item.boundAction();
        }
        catch (error) {
            throw this.props.errorHandler.report(error, 'ItemRenderer:doClick', []);
        }
    }
    handleClick(event) {
        event.preventDefault();
        event.stopPropagation();
        this.doClick();
    }
    doNegativeFeedback() {
        try {
            const notebook = this.props.notebook;
            const item = this.props.item;
            const em = notebook.experimentManager;
            if (item.feedback & 2) {
                em.reportFeedback(notebook, item, '<<negate-negative>>');
                item.feedback -= 2;
            }
            else {
                em.reportFeedback(notebook, item, '<<negative>>');
                item.feedback |= 2;
            }
            this.forceUpdate();
        }
        catch (error) {
            throw this.props.errorHandler.report(error, 'ItemRenderer:doNegativeFeedback', []);
        }
    }
    negativeFeedback(event) {
        event.preventDefault();
        event.stopPropagation();
        this.doNegativeFeedback();
    }
    doPositiveFeedback() {
        try {
            const notebook = this.props.notebook;
            const item = this.props.item;
            const em = notebook.experimentManager;
            if (item.feedback & 4) {
                em.reportFeedback(notebook, item, '<<negate-positive>>');
                item.feedback -= 4;
            }
            else {
                em.reportFeedback(notebook, item, '<<positive>>');
                item.feedback |= 4;
            }
            this.forceUpdate();
        }
        catch (error) {
            throw this.props.errorHandler.report(error, 'ItemRenderer:doPositiveFeedback', []);
        }
    }
    positiveFeedback(event) {
        event.preventDefault();
        event.stopPropagation();
        this.doPositiveFeedback();
    }
    doMessageFeedback() {
        let dialogResult = '';
        const onChange = (event) => {
            event.preventDefault();
            event.stopPropagation();
            dialogResult = event.currentTarget.value;
        };
        try {
            const body = (React.createElement(FeedbackDialogRenderer, { item: this.props.item, onChange: onChange }));
            const notebook = this.props.notebook;
            showTextDialog({
                title: 'Send Feedback',
                body: body,
                buttons: [Dialog.cancelButton(), Dialog.okButton({ label: 'Send' })],
            }).then((result) => {
                Promise.resolve(result.button.accept).then((ok) => {
                    try {
                        if (ok && dialogResult) {
                            notebook.experimentManager.reportFeedback(notebook, this.props.item, dialogResult);
                        }
                    }
                    catch (error) {
                        throw this.props.errorHandler.report(error, 'ItemRenderer:doMessageFeedback.then', [ok, dialogResult]);
                    }
                });
            });
        }
        catch (error) {
            throw this.props.errorHandler.report(error, 'ItemRenderer:doMessageFeedback', []);
        }
    }
    messageFeedback(event) {
        event.preventDefault();
        event.stopPropagation();
        this.doMessageFeedback();
    }
    onContextMenu(event) {
        try {
            const commands = new CommandRegistry();
            const contextMenu = new ContextMenu({ commands: commands });
            const item = this.props.item;
            commands.addCommand('info', {
                label: item.reportId === 'group' ? item.text : item.reportId,
                caption: item.action.title,
                className: 'julynter-context-info',
                execute: () => {
                    return;
                },
            });
            contextMenu.addItem({
                command: 'info',
                selector: '*',
            });
            contextMenu.addItem({
                type: 'separator',
                selector: '*',
            });
            commands.addCommand('action', {
                label: item.action.label,
                caption: item.action.title,
                execute: this.doClick.bind(this),
            });
            contextMenu.addItem({
                command: 'action',
                selector: '*',
            });
            commands.addCommand('copy', {
                label: 'Copy lint',
                caption: 'Copy lint text',
                execute: () => {
                    try {
                        const id = item.reportId;
                        const text = item.text;
                        const suggestion = item.suggestion || 'N/A';
                        Clipboard.copyToSystem(`ID: ${id}\nMessage: ${text}\nSuggestion: ${suggestion}`);
                    }
                    catch (error) {
                        throw this.props.errorHandler.report(error, 'ItemRenderer:onContextMenu.copy', [item]);
                    }
                },
            });
            contextMenu.addItem({
                command: 'copy',
                selector: '*',
            });
            let hideTitle = `Filter out lint ${item.reportId}`;
            let hideMessage = 'Are you sure you want to filter out' +
                `lints with the id "${item.reportId}"?`;
            let hideExample = (React.createElement("div", null,
                "Example of instance:",
                React.createElement("ul", { className: "julynter-feedback-ul" },
                    React.createElement("li", null,
                        "Message: ",
                        item.text),
                    React.createElement("li", null,
                        "Suggestion: ",
                        item.suggestion))));
            if (item.reportId === 'group') {
                hideTitle = `Filter out lint type ${item.text}`;
                hideMessage =
                    'Are you sure you want to filter out' +
                        `lints with the type "${item.text}"?`;
                hideExample = null;
            }
            commands.addCommand('hide', {
                label: 'Filter out similar lints',
                caption: 'Filter out this type of lint',
                execute: () => {
                    showDialog({
                        title: hideTitle,
                        body: (React.createElement("div", null,
                            React.createElement("div", null,
                                " ",
                                hideMessage,
                                " "),
                            hideExample)),
                        buttons: [Dialog.cancelButton(), Dialog.okButton({ label: 'Yes' })],
                    }).then((result) => {
                        Promise.resolve(result.button.accept).then((ok) => {
                            try {
                                if (ok && item.reportId !== 'group') {
                                    this.props.notebook.options.updateReport(item.reportId, false);
                                }
                                else if (ok) {
                                    this.props.notebook.options.updateType(item.reportType, false);
                                }
                            }
                            catch (error) {
                                throw this.props.errorHandler.report(error, 'ItemRenderer:onContextMenu.hide', [ok, item]);
                            }
                        });
                    });
                },
            });
            contextMenu.addItem({
                command: 'hide',
                selector: '*',
            });
            if (this.props.item.feedback) {
                contextMenu.addItem({
                    type: 'separator',
                    selector: '*',
                });
                const neg = item.feedback & 2 ? 'Undo' : 'Send';
                commands.addCommand('negative', {
                    label: `${neg} negative feedback`,
                    caption: `${neg} feedback indicating that you do not like this lint`,
                    execute: this.doNegativeFeedback.bind(this),
                });
                contextMenu.addItem({
                    command: 'negative',
                    selector: '*',
                });
                const pos = item.feedback & 4 ? 'Undo' : 'Send';
                commands.addCommand('positive', {
                    label: `${pos} positive feedback`,
                    caption: `${pos} feedback indicating that you like this lint`,
                    execute: this.doPositiveFeedback.bind(this),
                });
                contextMenu.addItem({
                    command: 'positive',
                    selector: '*',
                });
                commands.addCommand('feedback', {
                    label: 'Send feedback',
                    caption: 'Send a textual feedback',
                    execute: this.doMessageFeedback.bind(this),
                });
                contextMenu.addItem({
                    command: 'feedback',
                    selector: '*',
                });
            }
            contextMenu.open(event);
            event.preventDefault();
            event.stopPropagation();
        }
        catch (error) {
            throw this.props.errorHandler.report(error, 'ItemRenderer:onContextMenu', []);
        }
    }
    render() {
        try {
            const item = this.props.item;
            if (!item.visible || item.filteredOut) {
                return null;
            }
            const reportDivClass = item.hasParent
                ? 'julynter-report-div julynter-has-parent'
                : 'julynter-report-div';
            let twistButton = null;
            let fontSize = 'julynter-normal-size';
            let feedbackDiv = null;
            if (item.collapsed) {
                fontSize = 'julynter-title-size';
                twistButton = (React.createElement("div", { className: "julynter-collapse-button" },
                    React.createElement("div", { className: "julynter-twist-placeholder" }, "placeholder"),
                    React.createElement("div", { className: "julynter-rightarrow-img julynter-arrow-img" })));
            }
            else if (item.collapsed === false) {
                fontSize = 'julynter-title-size';
                twistButton = (React.createElement("div", { className: "julynter-collapse-button" },
                    React.createElement("div", { className: "julynter-twist-placeholder" }, "placeholder"),
                    React.createElement("div", { className: "julynter-downarrow-img julynter-arrow-img" })));
            }
            if (item.feedback &&
                this.props.notebook.experimentManager.config.enabled) {
                const negativeClass = 'julynter-feedback-icon jp-julynter-feedback-negative-icon' +
                    (item.feedback & 2 ? ' julynter-feedback-icon-selected' : '');
                const positiveClass = 'julynter-feedback-icon jp-julynter-feedback-positive-icon' +
                    (item.feedback & 4 ? ' julynter-feedback-icon-selected' : '');
                feedbackDiv = (React.createElement("div", { className: "julynter-feedback" },
                    React.createElement("span", { className: "julynter-feedback-text" }, "Experiment Feedback:"),
                    React.createElement("div", { className: "julynter-feedback-buttons" },
                        React.createElement("div", { className: "julynter-feedback-button", onClick: this.negativeFeedback.bind(this) },
                            React.createElement("div", { role: "text", "aria-label": "I do not like this lint", title: "I do not like this lint", className: negativeClass })),
                        React.createElement("div", { className: "julynter-feedback-button", onClick: this.positiveFeedback.bind(this) },
                            React.createElement("div", { role: "text", "aria-label": "I like this lint", title: "I like this lint", className: positiveClass })),
                        React.createElement("div", { className: "julynter-feedback-button", onClick: this.messageFeedback.bind(this) },
                            React.createElement("div", { role: "text", "aria-label": "Send a text feedback about this lint", title: "Send a text feedback about this lint", className: "julynter-feedback-icon jp-julynter-feedback-text-icon" })))));
            }
            const reportPromptClass = 'julynter-report-prompt ' + fontSize;
            return (React.createElement("li", { onClick: this.handleClick },
                React.createElement("div", { className: "julynter-entry-holder", title: this.props.item.suggestion, onContextMenu: this.onContextMenu.bind(this) },
                    twistButton,
                    React.createElement("div", { className: reportDivClass },
                        React.createElement("div", { className: reportPromptClass },
                            React.createElement("div", null, item.text),
                            feedbackDiv)))));
        }
        catch (error) {
            throw this.props.errorHandler.report(error, 'ItemRenderer:render', []);
        }
    }
}
