import * as React from 'react';
import { Widget } from '@lumino/widgets';
import { showDialog, Dialog, showErrorMessage } from '@jupyterlab/apputils';
import { PathExt } from '@jupyterlab/coreutils';
import { ERROR_TYPES_MAP } from '../../linter/reports';
import { ErrorTypeKeys } from '../../linter/interfaces';
import { createJulynterConfigWidget } from './julynterconfigwidget';
export class ToolbarRenderer extends React.Component {
    constructor(props) {
        super(props);
        this.toggle = this.toggle.bind(this);
        this.toggleMode = this.toggleMode.bind(this);
        this.changeRequirements = this.changeRequirements.bind(this);
    }
    toggle(key) {
        try {
            const options = this.props.notebook.options;
            options.updateType(key, !options.checkType(key));
        }
        catch (error) {
            throw this.props.errorHandler.report(error, 'ToolbarRenderer:toggle', [
                key,
            ]);
        }
    }
    toggleMode() {
        try {
            let mode = this.props.notebook.options.checkMode();
            if (mode === 'list') {
                mode = 'cell';
            }
            else if (mode === 'cell') {
                mode = 'type';
            }
            else {
                mode = 'list';
            }
            this.props.notebook.options.updateMode(mode);
        }
        catch (error) {
            throw this.props.errorHandler.report(error, 'ToolbarRenderer:toggleMode', []);
        }
    }
    changeRequirements() {
        try {
            const options = this.props.notebook.options;
            return showDialog({
                title: 'Set requirements file',
                body: new RequirementsHandler(options.checkRequirements()),
                focusNodeSelector: 'input',
                buttons: [Dialog.cancelButton(), Dialog.okButton({ label: 'Set' })],
            }).then((result) => {
                if (!result.value) {
                    return;
                }
                const name = result.value;
                if (name.length === 0 || name.includes(':')) {
                    void showErrorMessage('Set Requirements Error', Error(`"${result.value}" is not a valid name for a file. ` +
                        'Names must have nonzero length, ' +
                        'and cannot include ":"'));
                    return null;
                }
                options.updateRequirements(name);
            });
        }
        catch (error) {
            throw this.props.errorHandler.report(error, 'ToolbarRenderer:changeRequirements', []);
        }
    }
    configure() {
        try {
            this.props.labShell.add(createJulynterConfigWidget(this.props.tracker, this.props.handlers, this.props.config, this.props.notebook, this.props.errorHandler), 'main');
        }
        catch (error) {
            throw this.props.errorHandler.report(error, 'ToolbarRenderer:configure', []);
        }
    }
    render() {
        try {
            let i = 0;
            const listing = ErrorTypeKeys.map((key) => {
                const element = ERROR_TYPES_MAP[key];
                const toggleClass = element.icon +
                    ' ' +
                    (this.props.notebook.options.checkType(key)
                        ? 'julynter-toolbar-icon-selected'
                        : 'julynter-toolbar-icon');
                const buttonClass = 'julynter-toolbar-button';
                const label = element.label;
                return (React.createElement("div", { key: `toolbar-${key}-${i++}`, className: buttonClass, onClick: () => this.toggle(key) },
                    React.createElement("div", { role: "text", "aria-label": label, title: label, className: toggleClass })));
            });
            let toggleClass = 'julynter-toolbar-icon';
            const mode = this.props.notebook.options.checkMode();
            if (mode === 'list') {
                toggleClass += ' julynter-toolbar-list-icon';
            }
            else if (mode === 'cell') {
                toggleClass += ' julynter-toolbar-cell-icon';
            }
            else if (mode === 'type') {
                toggleClass += ' julynter-toolbar-type-icon';
            }
            const modeToggle = (React.createElement("div", { key: "toolbar-mode", className: "julynter-toolbar-button", onClick: this.toggleMode },
                React.createElement("div", { role: "text", "aria-label": "Alternate Mode", title: "Alternate Mode", className: toggleClass })));
            const reqConfig = (React.createElement("div", { key: "toolbar-req", className: "julynter-toolbar-button", onClick: this.changeRequirements },
                React.createElement("div", { role: "text", "aria-label": "Change requirements location", title: "Change requirements location", className: "julynter-toolbar-icon julynter-toolbar-rename-icon" })));
            const configure = (React.createElement("div", { key: "toolbar-config", className: "julynter-toolbar-button", onClick: this.configure.bind(this) },
                React.createElement("div", { role: "text", "aria-label": "Configure Julynter", title: "Configure Julynter", className: "julynter-toolbar-icon julynter-toolbar-config-icon" })));
            return (React.createElement("div", null,
                React.createElement("div", { className: 'julynter-toolbar' },
                    listing,
                    modeToggle,
                    reqConfig,
                    configure)));
        }
        catch (error) {
            throw this.props.errorHandler.report(error, 'ToolbarRenderer:render', []);
        }
    }
}
/**
 * A widget used to rename a file.
 */
class RequirementsHandler extends Widget {
    /**
     * Construct a new "rename" dialog.
     */
    constructor(oldPath) {
        const body = document.createElement('div');
        const existingLabel = document.createElement('label');
        existingLabel.textContent = 'Requirements file';
        const existingPath = document.createElement('span');
        existingPath.textContent = oldPath;
        const nameTitle = document.createElement('label');
        nameTitle.textContent = 'New Name';
        nameTitle.className = 'jp-new-name-title';
        const name = document.createElement('input');
        body.appendChild(existingLabel);
        body.appendChild(existingPath);
        body.appendChild(nameTitle);
        body.appendChild(name);
        super({ node: body });
        this.addClass('jp-FileDialog');
        const ext = PathExt.extname(oldPath);
        const value = (this.inputNode.value = PathExt.basename(oldPath));
        this.inputNode.setSelectionRange(0, value.length - ext.length);
    }
    /**
     * Get the input text node.
     */
    get inputNode() {
        return this.node.getElementsByTagName('input')[0];
    }
    /**
     * Get the value of the widget.
     */
    getValue() {
        return this.inputNode.value;
    }
}
